import threading
# import multiprocessing
from queue import Queue
# import asyncio
import logging
from datetime import datetime
import time
from typing import Callable, Any, List, Dict

class BackgroundTask:
    """백그라운드 작업을 관리하는 클래스"""
    def __init__(self):
        self.task_queue = Queue()
        self.results = {}
        self.is_running = True
        self.workers: List[threading.Thread] = []
        
        # 로깅 설정
        self.logger = self._setup_logger()
        
    def _setup_logger(self) -> logging.Logger:
        """로깅 설정"""
        logger = logging.getLogger('BackgroundTask')
        logger.setLevel(logging.INFO)
        
        handler = logging.StreamHandler()
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        )
        handler.setFormatter(formatter)
        logger.addHandler(handler)
        
        return logger
    
    def add_task(self, task_func: Callable, *args, **kwargs) -> str:
        """새로운 작업 추가"""
        task_id = str(time.time())
        self.task_queue.put((task_id, task_func, args, kwargs))
        self.logger.info(f"Task {task_id} added to queue")
        return task_id
    
    def start_workers(self, num_workers: int = 4):
        """워커 스레드 시작"""
        for _ in range(num_workers):
            worker = threading.Thread(target=self._worker_thread)
            worker.daemon = True
            worker.start()
            self.workers.append(worker)
            
    def _worker_thread(self):
        """워커 스레드 실행 함수"""
        while self.is_running:
            try:
                # 큐에서 작업 가져오기
                task_id, task_func, args, kwargs = self.task_queue.get(timeout=1)
                
                # 작업 실행
                self.logger.info(f"Starting task {task_id}")
                start_time = time.time()
                
                try:
                    result = task_func(*args, **kwargs)
                    self.results[task_id] = {
                        'status': 'completed',
                        'result': result,
                        'error': None,
                        'completion_time': datetime.now(),
                        'execution_time': time.time() - start_time
                    }
                    self.logger.info(f"Task {task_id} completed successfully")
                    
                except Exception as e:
                    self.results[task_id] = {
                        'status': 'failed',
                        'result': None,
                        'error': str(e),
                        'completion_time': datetime.now(),
                        'execution_time': time.time() - start_time
                    }
                    self.logger.error(f"Task {task_id} failed: {str(e)}")
                
                finally:
                    self.task_queue.task_done()
                    
            except TimeoutError:
                continue
    
    def get_result(self, task_id: str) -> Dict:
        """작업 결과 조회"""
        return self.results.get(task_id)
    
    def stop(self):
        """백그라운드 작업 중지"""
        self.is_running = False
        for worker in self.workers:
            worker.join()
        self.logger.info("All workers stopped")