import os
import requests
import json
from dotenv import load_dotenv

load_dotenv()

class BandTask:
    def __init__(self):
        self.access_token = os.getenv("NAVER_BAND_ACCESS_TOKEN")
        self.band_key = os.getenv("NAVER_BAND_KEY")
        
    def _api_call(self, api_path: str, params: dict = None, method: str = 'get') -> dict:
        if params is None:
            params = {}
        params['access_token'] = self.access_token
        params['band_key'] = self.band_key
        print(f'[api call] api_path : {api_path}, params : {params}')
        
        if method == 'get':
            api_call_result = requests.get('https://openapi.band.us' + api_path, params=params)
        elif method == 'post':
            api_call_result = requests.post('https://openapi.band.us' + api_path, params)
        else:
            print(f'not support method. method: {method}')
            return None

        if not api_call_result.ok:
            print(f'api call fail. api call result : {api_call_result}, text : {api_call_result.text}')
            return None

        api_call_result_json = json.loads(api_call_result.text)
        if 'result_code' not in api_call_result_json or 'result_data' not in api_call_result_json:
            print(f'api call result structure malformed. api call result json : {api_call_result_json}')
            return None

        return api_call_result_json['result_data']

    def get_album_photos(self,  photo_album_key: str) -> dict:
        return self._api_call('/v2/band/album/photos',
                              params={ 'photo_album_key': photo_album_key}, )

    def get_albums(self, band_key: str) -> dict:
        return self._api_call('/v2/band/albums', )

    def delete_post_comments(self,  post_key: str, comment_key: str):
        return self._api_call('/v2/band/post/comment/remove',
                              params={'post_key': post_key, 'body': comment_key},
                              method='post')

    def create_post_comments(self,  post_key: str, body: str):
        return self._api_call('/v2/band/post/comment/create',
                              params={ 'post_key': post_key, 'body': body},
                              method='post')

    def get_post_comments(self,  post_key: str):
        return self._api_call('/v2/band/post/comments',
                              params={'post_key': post_key}, )

    def delete_post(self,  post_key: str):
        return self._api_call('/v2/band/post/remove',
                              params={'post_key': post_key},
                              method='post')

    def create_post(self,  content: str):
        return self._api_call('/v2.2/band/post/create',
                              params={'content': content},
                              method='post')

    def get_post(self,  post_key: str):
        return self._api_call('/v2.1/band/post', params={  'post_key': post_key})

    def get_posts(self):
        return self._api_call('/v2/band/posts', params={ 'locale': "ko_KR"})

    def get_bands(self) -> dict:
        return self._api_call('/v2.1/bands', )

    def get_profile(self) -> dict:
        return self._api_call('/v2/profile')
    
    def upload_photo(self, file_path: str) -> dict:
        """
        밴드에 사진을 업로드합니다.
        :param file_path: 업로드할 이미지 파일 경로
        :return: 업로드된 사진 정보를 담은 딕셔너리
        """
        with open(file_path, 'rb') as f:
            files = {'photo': f}
            params = {
                'access_token': self.access_token,
                'band_key': self.band_key
            }
            response = requests.post('https://openapi.band.us/v2.2/band/photo/upload', 
                                   params=params,
                                   files=files)
            
            if not response.ok:
                print(f'photo upload fail. response: {response}, text: {response.text}')
                return None
                
            result = json.loads(response.text)
            if 'result_code' not in result or 'result_data' not in result:
                print(f'photo upload result structure malformed. result: {result}')
                return None
                
            return result['result_data']
            
    def create_post_with_photo(self, content: str, photo_keys: list) -> dict:
        """
        사진이 포함된 게시물을 생성합니다.
        :param content: 게시물 내용
        :param photo_keys: upload_photo()로 얻은 photo_key 리스트
        :return: 생성된 게시물 정보
        """
        params = {
            'content': content,
            'photo_keys': ','.join(photo_keys)
        }
        return self._api_call('/v2.2/band/post/create', 
                            params=params,
                            method='post')