
import os
from ..ha_constant import HaConstant
from pydrive.auth import GoogleAuth
from pydrive.drive import GoogleDrive
from ..tasks.file_task import FileTask
class GoogleDriveTask:      
    def __init__(self):
        print("GoogleDriveTask init")
        self.drive = self.authenticate_google_drive()
        self.folder_id = HaConstant.google_drive_folder_id
        self.task_file = FileTask()

    def authenticate_google_drive(self):
        """Google Drive API 인증을 처리하는 함수"""
        try:
            gauth = GoogleAuth()
            gauth.LoadCredentialsFile(HaConstant.google_drive_credential_path)
            if gauth.credentials is None:   
                gauth.LocalWebserverAuth()
            elif gauth.access_token_expired:
                gauth.Refresh()
            else:
                gauth.Authorize()

            gauth.SaveCredentialsFile(HaConstant.google_drive_credential_path)

            drive = GoogleDrive(gauth)

            self.drive = drive
            return drive
        except Exception as e:
            print(f"Error: {e}")


       

    def upload_file(self, filename, audio_id):
        """
        파일을 Google Drive에 업로드하는 함수
        
        Parameters:
        filename (str): 업로드될 파일의 이름
        filepath (str): 업로드할 파일의 경로
        mimetype (str): 파일의 MIME 타입 (예: 'image/jpeg', 'application/pdf')
        
        Returns:
        dict: 업로드된 파일의 정보
        """
        try:

            # Google Drive API 인증
             #위에서 복사한 구글드라이브 폴더의 id
            tfile = f"{filename}.txt"
            if self.get_file_id(tfile, parent_id=self.folder_id):
                print(f'파일이 이미 존재합니다. 업로드 건너뜀: {tfile}')
                return
            file = self.drive.CreateFile({'title': audio_id, 'parents': [{'id': self.folder_id}],})
            file.SetContentFile(tfile)
            file.Upload()
        
            return file
            
        except Exception as e:
            print(f'업로드 중 오류가 발생했습니다: {str(e)}')
            return None




    def create_folder(self, folder_name, parent_id=None):
        """
        Google Drive에 폴더를 생성하는 함수
        
        Parameters:
        folder_name (str): 생성할 폴더 이름
        parent_id (str): 상위 폴더 ID (없으면 root에 생성)
        
        Returns:
        str: 생성된 폴더의 ID
        """
        try:
            folder_metadata = {
                'title': folder_name,
                'mimeType': 'application/vnd.google-apps.folder'
            }
            
            if parent_id:
                folder_metadata['parents'] = [{'id': parent_id}]
                
            folder = self.drive.CreateFile(folder_metadata)
            folder.Upload()
            
            return folder['id']
        except Exception as e:
            print(f'폴더 생성 중 오류 발생: {str(e)}')
            return None

    def get_file_id(self, file_name, parent_id=None):
        """
        파일 이름으로 Google Drive의 파일 ID를 찾는 함수
        
        Parameters:
        file_name (str): 찾을 파일 이름
        parent_id (str): 검색할 상위 폴더 ID (없으면 root에서 검색)
        
        Returns:
        str: 찾은 파일의 ID
        """
        # print(f"file_name: {file_name} parent_id: {parent_id}")
        try:
            query = f"title='{file_name}' and mimeType!='application/vnd.google-apps.folder'"
            if parent_id:
                query += f" and '{parent_id}' in parents"
                
            file_list = self.drive.ListFile({'q': query}).GetList()
            
            if file_list:
                return file_list[0]['id']
            return None
            
        except Exception as e:
            print(f'파일 검색 중 오류 발생: {str(e)}')
            return None

    def upload_folder(self, local_folder_path, parent_folder_id=HaConstant.google_drive_folder_id):
        """
        로컬 폴더를 Google Drive에 업로드하는 함수
        
        Parameters:
        local_folder_path (str): 업로드할 로컬 폴더 경로
        parent_folder_id (str): Google Drive의 상위 폴더 ID (없으면 root에 업로드)
        
        Returns:
        str: 생성된 최상위 폴더의 ID
        """

        try:
            # 폴더 이름 추출
            folder_name = os.path.basename(local_folder_path)
            
            # 폴더가 이미 존재하는지 확인
            existing_folder_id = self.get_folder_id(folder_name, parent_folder_id)
            if existing_folder_id:
                print(f'폴더가 이미 존재합니다. 기존 폴더 사용: {folder_name}')
                folder_id = existing_folder_id
            else:
                # Google Drive에 새 폴더 생성
                folder_id = self.create_folder(folder_name, parent_folder_id)
                print(f'새 폴더 생성됨: {folder_name}')
            
            # 폴더 내의 모든 항목을 순회
            for item in os.listdir(local_folder_path):
                item_path = os.path.join(local_folder_path, item)
                
                if os.path.isfile(item_path):
                    # 파일이 이미 존재하는지 확인
                    if self.get_file_id(item, parent_id=folder_id):
                        print(f'파일이 이미 존재합니다. 업로드 건너뜀: {item}')
                        continue
                    
                    # 파일인 경우 업로드
                    file = self.drive.CreateFile({
                        'title': item,
                        'parents': [{'id': folder_id}]
                    })
                    file.SetContentFile(item_path)
                    file.Upload()
                    print(f'파일 업로드 완료: {item}')
                    
                elif os.path.isdir(item_path):
                    # 폴더인 경우 재귀적으로 처리
                    self.upload_folder(item_path, folder_id)
            
            return folder_id
            
        except Exception as e:
            print(f'폴더 업로드 중 오류 발생: {str(e)}')
            return None

    def get_folder_id(self, folder_name, parent_id=None):
        """
        폴더 이름으로 Google Drive의 폴더 ID를 찾는 함수
        
        Parameters:
        folder_name (str): 찾을 폴더 이름
        parent_id (str): 검색할 상위 폴더 ID (없으면 root에서 검색)
        
        Returns:
        str: 찾은 폴더의 ID
        """
        try:
            query = f"title='{folder_name}' and mimeType='application/vnd.google-apps.folder'"
            if parent_id:
                query += f" and '{parent_id}' in parents"
                
            file_list = self.drive.ListFile({'q': query}).GetList()
            
            if file_list:
                return file_list[0]['id']
            return None
            
        except Exception as e:
            print(f'폴더 검색 중 오류 발생: {str(e)}')
            return None          
        

    def delete_file(self, audio_id):

        folder_id = self.get_folder_id(audio_id, parent_id=self.folder_id)
        print(f"folder_id: {audio_id} {folder_id}")

        if folder_id:
            exist_embeddings = self.task_file.exist_embed_file(audio_id=audio_id)
            if exist_embeddings:
                self.task_file.delete_folder(folder_path=f"{HaConstant.embed_dir}/{audio_id}")
                print(f'embeddings 파일 삭제 완료: {audio_id}')
            else:
                print(f'embeddings 파일을 찾을 수 없습니다: {audio_id}')
       
        file_id = self.get_file_id(audio_id, parent_id=self.folder_id)
        print(f"file_id: {file_id}")
        if file_id:
            exist_trans_file = self.task_file.exist_text_file(audio_id=audio_id)
            if exist_trans_file:
                self.task_file.delete_text_file(audio_id=audio_id)
                print(f'파일 삭제 완료: {audio_id}')
            else:
                print(f'파일을 찾을 수 없습니다: {audio_id}')

